****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: 01_mp4_10.sas  
*
* Created (mm/dd/yy): 02/14/2014
* Last modified: 02/14/2014
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Mini-Sentinel : 	Continuous and Group Sequential Analysis with Self-Control Design 
*										PROMPT (Prospective Routine Observational Monitoring Program Tool) 
*
*										Create summary tables for drug of interests and adverse events.
*										See documentation for macro and parameter details.
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;

 %MACRO mp410 (exposure_period_cd=,
							ENROLGAP=45,
							QUERYFROM="01JAN1970"d,
							QUERYTO="31AUG2013"d,
							QUERYFILE=query.sas7bdat,    
							INCQUERYFILE=incquery.sas7bdat,
							EVENTFILE=event.sas7bdat,
							INCEVENTFILE=incevent.sas7bdat,
							QUERY_CODES_FILE=query_codes.sas7bdat,
							EVENT_CODES_FILE=event_codes.sas7bdat,
							GROUPS_MAPPING_FILE=groups_mapping.sas7bdat,
							AGE_GROUPS_FILE=age_groups.sas7bdat,
							EXPOSURE_PERIODS_FILE=exposure_periods.sas7bdat,
							LKP_FOR_ALL_CODES_FILE=lkp_for_all_codes.sas7bdat,
							tmp=WORK,
							tmp2=WORK
							);

%LET DPID=%LOWCASE(&DPID.);
%LET SITEID=%LOWCASE(&SITEID.);
%LET REQUESTID=%LOWCASE(&REQUESTID.);


 *For Custom Report -- Proc PrintTo;

DATA _NULL_;
        call symput("DateStamp",put(datepart(DATETIME()),date9.));
        call symput("TimeStamp",compress( "d"||substr( put(timepart(DATETIME()),hhmm6.),1,3)||
										 "h"||substr( put(timepart(DATETIME()),hhmm6.),5,2)||"m") );
RUN;

	
 DATA _NULL_;
  LENGTH exposure_period_cd $ 300;
  exposure_period_cd="&exposure_period_cd.";
   * Replace commas with blanks in case they were present in the list;
   exposure_period_cd= TRANSLATE(exposure_period_cd, " " , "," );

 * Remove multiple blanks and replace blank with _;
   exposure_period_str= TRANSLATE(TRIM(COMPBL(exposure_period_cd)), "_" , " " );

   CALL SYMPUT("exposure_period_cd", TRIM(exposure_period_cd));
   CALL SYMPUT("exposure_period_str", TRIM(exposure_period_str));
 RUN;

 %PUT exposure_period_cd=&exposure_period_cd.;
 %PUT exposure_period_str=&exposure_period_str.;


%LET out_tbl_prefix=&DPID.&SITEID._ep_&exposure_period_str.;
%LET out_rpt_prefix=&DPID.&SITEID._&REQUESTID.;

%PUT out_tbl_prefix=&out_tbl_prefix.;
%PUT out_rpt_prefix=&out_rpt_prefix.;


   PROC PRINTTO LOG="&msoc.&out_rpt_prefix..log" new;
   RUN;

   PROC PRINTTO PRINT="&msoc.&out_rpt_prefix._valid_rpt.lst" new;
   RUN;

   /*Set macro variables for Query dates*/
data _NULL_;
   temp=DATETIME();
   call symput("START",temp);
   call symput("STARTDATE",put(datepart(temp),date9.));
   call symput("STARTTIME",put(timepart(temp),time4.));
run;

Data _null_ ;
 QUERYFROM=&QUERYFROM.;
 QUERYTO=&QUERYTO.;
   Call Symput("QUERYFROMc",put(QUERYFROM,mmddyy10.));
   Call Symput("QUERYTOc",put(QUERYTO,mmddyy10.)); 
run;

%PUT &STARTDATE.;
%PUT &STARTTIME.;
%PUT &QUERYFROMc;
%PUT &QUERYTOc;


ODS ESCAPECHAR="^";
OPTIONS ORIENTATION=LANDSCAPE LEFTMARGIN=1IN RIGHTMARGIN=1IN TOPMARGIN=1IN BOTTOMMARGIN=1IN;

%LET QUERY=_QUERY0;
%LET INCQUERY=_INCQUERY0;
%LET EVENT=_EVENT0;
%LET INCEVENT=_INCEVENT0;

 %LET QUERY_CODES=_QUERY_CODES0;
 %LET EVENT_CODES=_EVENT_CODES0;

%LET GROUPS_MAPPING=_GROUPS_MAPPING0;
%LET AGE_GROUPS=_AGE_GROUPS0;
%LET EXPOSURE_PERIODS=_EXPOSURE_PERIODS0;
%LET LKP_FOR_ALL_CODES=_LKP_FOR_ALL_CODES0;

/*
%LET EXPOSURE_PERIODS=INFOLDER.EXPOSURE_PERIODS;
%LET QUERY_CODES=INFOLDER.QUERY_CODES;
%LET QUERY=INFOLDER.QUERY;
%LET INCQUERY=INFOLDER.INCQUERY;
%LET EVENT=INFOLDER.EVENT;
%LET INCEVENT=INFOLDER.INCEVENT;
%LET GROUPS_MAPPING=INFOLDER.GROUPS_MAPPING;
%LET AGE_GROUPS=INFOLDER.AGE_GROUPS;
 %LET EVENT_CODES=INFOLDER.EVENT_CODES;
%LET LKP_FOR_ALL_CODES=INFOLDER.LKP_FOR_ALL_CODES;
*/

%MACRO IMPORTFILES(var1,var2,var3);

%LET VAR1=%lowcase(&var1.);

   %IF %INDEX(%UPCASE("&VAR1."),CPORT) %THEN %DO;

proc cimport infile="&infolder.&VAR1." library=infolder memtype=data;
run;

   %END;
   %LET &VAR2.=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);
   %LET VAR1=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);

data &var2.;
   set infolder.&var1.;
   keep &var3.;
run;

%MEND;

* Import mapping and lookup tables supplied with the package;

%IMPORTFILES(&QUERYFILE.,&QUERY.,_ALL_);

%IMPORTFILES(&EVENTFILE.,&EVENT.,_ALL_);

%IMPORTFILES(&QUERY_CODES_FILE.,&QUERY_CODES.,_ALL_);
%IMPORTFILES(&EVENT_CODES_FILE., &EVENT_CODES.,_ALL_);

%IMPORTFILES(&AGE_GROUPS_FILE.,&AGE_GROUPS.,_ALL_);
%IMPORTFILES(&GROUPS_MAPPING_FILE.,&GROUPS_MAPPING.,_ALL_);

%IMPORTFILES(&GROUPS_MAPPING_FILE.,&GROUPS_MAPPING.,_ALL_);
%IMPORTFILES(&AGE_GROUPS_FILE.,&AGE_GROUPS.,_ALL_);
%IMPORTFILES(&EXPOSURE_PERIODS_FILE.,&EXPOSURE_PERIODS.,_ALL_);
%IMPORTFILES(&LKP_FOR_ALL_CODES_FILE.,&LKP_FOR_ALL_CODES.,_ALL_);
 
   
%MACRO WRAPPER();
   %IF %INDEX(%UPCASE("&INCQUERYFILE."),.) %THEN %DO;
      %IMPORTFILES(&INCQUERYFILE.,&INCQUERY.,_ALL_);
   %END;
   %ELSE %DO;

data &INCQUERY.;
   if 0 then set &QUERY.;
   stop;
run;
   				%END;
   %IF %INDEX(%UPCASE("&INCEVENTFILE."),.) %THEN 
			%DO;
      %IMPORTFILES(&INCEVENTFILE.,&INCEVENT.,_ALL_); 
   			%END;
   %ELSE 
			%DO;

data &INCEVENT.;
   if 0 then set &EVENT.;
   stop;
run;
   				%END;

%MEND WRAPPER;

%wrapper();

PROC SQL  NOPRINT;
     SELECT 
		MAX((LIBNAME ="INDATA" AND ENGINE  IN ("V7", "V8", "V9"))) 
	   ,MAX((LIBNAME ="&tmp2." AND ENGINE  IN ("V7", "V8", "V9")))
			INTO :indata_is_sas, :tmp_for_join_is_sas
	
	   FROM SASHELP.VLIBNAM
         WHERE LIBNAME IN ("INDATA", "&tmp2." )
	;
QUIT;

%LET indata_is_sas=&indata_is_sas. ;
%LET tmp_for_join_is_sas=&tmp_for_join_is_sas.;

%PUT indata_is_sas=&indata_is_sas. ;
%PUT tmp_for_join_is_sas=&tmp_for_join_is_sas.;

* Several steps below do some initial processing of mapping and lookup table;
* These included setting some defaults, joining some of the tables together and creating;
* macro variables for min and max dates which will be used latter in the program.

 * Set age group defaults and create AGE_INTERVAL_TYPE which will be used to calculate age
	and create variable AGE_GRP_DSC;

DATA &tmp.._AGE_GROUPS;
	SET &AGE_GROUPS.;
	  LENGTH AGE_MAX_C $ 9 AGE_INTERVAL_TYPE $ 8 AGE_GRP_DSC $ 20 ;

	 IF AGE_MIN=. THEN AGE_MIN=0;
	 IF AGE_MAX=. THEN AGE_MAX=200;
	 IF AGE_UNIT=" " THEN AGE_UNIT="Y";

	 AGE_INTERVAL_TYPE=UPCASE(AGE_UNIT);

    AGE_INTERVAL_TYPE=TRANWRD(AGE_INTERVAL_TYPE, "Y","YEAR");
    AGE_INTERVAL_TYPE=TRANWRD(AGE_INTERVAL_TYPE, "D","DAY");
    AGE_INTERVAL_TYPE=TRANWRD(AGE_INTERVAL_TYPE, "W","WEEK");
    AGE_INTERVAL_TYPE=TRANWRD(AGE_INTERVAL_TYPE, "Q","QTR");
    AGE_INTERVAL_TYPE=TRANWRD(AGE_INTERVAL_TYPE, "M","MONTH"); 

   IF AGE_MAX >= 130 THEN  AGE_MAX_C = " >=";
   ELSE 									AGE_MAX_C = " - " || COMPRESS(PUT(AGE_MAX,8.)) ;

	 AGE_GRP_DSC=COMPRESS(PUT(AGE_MIN,8.)) || AGE_MAX_C
								|| " " | | TRIM(AGE_INTERVAL_TYPE) || "S";
		DROP AGE_MAX_C;
RUN;

PROC SQL;
 CREATE TABLE &tmp.._EXPOSURE_PERIODS AS	
 	SELECT  EXPOSURE_PERIOD_CD
				,EXPOS_PERIOD_STRT_DT 
				,EXPOS_PERIOD_END_DT 
			FROM &EXPOSURE_PERIODS.
			 WHERE EXPOSURE_PERIOD_CD IN (&exposure_period_cd.)
			     AND LOWCASE(DPID)=LOWCASE("&DPID.")
				 AND LOWCASE(SITEID)=LOWCASE("&SITEID.")
	;
QUIT;

 * Set default for WASHTYP and WASHPER. Set non-principal PDX to " " ;
*  Make values of character variables uppercase. ;

DATA &tmp.._QUERY;
 SET &QUERY.;

   WASHTYP=UPCASE(WASHTYP);

  IF WASHTYP = " " THEN WASHTYP = "MULT"; 
  IF WASHPER = .  	 THEN WASHPER = 0;
      
RUN;

* Combine tables with QUERY and INCQUERY tables into a single table;
* Create flags QUERY_FLAG and INCQUERY_FLAG that indicates the role of the SUBGROUP. 
* The SUBGROUP for a study group identifies the set of NDC, PX or DX codes; 
* The mapping of these codes to SUBGROUPS is defined in the separate table: QUERY_CODES;

PROC SQL ;
  CREATE TABLE &tmp.._QUERY_INCQUERY AS
    SELECT  COALESCE(A.QUERY_GRP,B.QUERY_GRP) AS QUERY_GRP
				,COALESCE(A.SUBGROUP,B.SUBGROUP)   AS SUBGROUP
				,A.WASHTYP
				,A.WASHPER
				,A.EXCL_MEDS_ON_SAME_DAY
				,(A.QUERY_GRP IS NOT NULL ) 	AS QUERY_FLAG
				,(B.QUERY_GRP IS NOT NULL )	AS INCQUERY_FLAG


	FROM 		&tmp.._QUERY 	A
	FULL JOIN &INCQUERY.		B
		ON A.QUERY_GRP=B.QUERY_GRP
	 AND  A.SUBGROUP=B.SUBGROUP
	    ORDER BY  QUERY_GRP
						,SUBGROUP
	;
QUIT;

 * Set default for WASHTYP and WASHPER. Set non-principal PDX to " ". ;
 *  Make values of character variables uppercase. ;

DATA &tmp.._EVENT;
 SET &EVENT.;
   	WASHTYP=UPCASE(WASHTYP);
	ENCTYPE=UPCASE(ENCTYPE);
  	PDX=UPCASE(PDX);

	IF PDX NE "P" THEN PDX=" ";

  IF WASHTYP = " " THEN WASHTYP = "MULT"; 
  IF WASHPER = .  	 THEN WASHPER = 0;
RUN;


DATA &tmp.._INCEVENT;
 SET &INCEVENT.;

    ENCTYPE=UPCASE(ENCTYPE);
  	PDX=UPCASE(PDX);
	IF PDX NE "P" THEN PDX=" ";
RUN;

* Combine tables with EVENT and INCEVENT tables into a single table;
* Create flags EVENT_FLAG and INCEVENT_FLAG that indicates the role of the SUBGROUP. 
* The tables EVENT and INCEVENT allow NULL values that must be interpreted as any allowed values;
* For example, ENCTYPE IS NULL in one of these tables means that all five possible values of ENCTYPE are allowed;
* The RIGHT JOINs to the LKP_FOR_ALL_CODES create a table which has explicit set of all possible codes;
* The SUBGROUP for a study group identifies the set of NDC, PX or DX codes; 
* The mapping of these codes to SUBGROUPS is defined in the separate table: EVENT_CODES;

* The query gives warning message to the log about Cartesian product.;
* This is intentional. Ignore it. ;


PROC SQL ;
  CREATE TABLE &tmp.._EVENT_INCEVENT AS
    SELECT 
				 COALESCE(A.EVENT_GRP,B.EVENT_GRP) 				AS EVENT_GRP
				,(A.EVENT_GRP IS NOT NULL ) 								AS EVENT_FLAG
				,(B.EVENT_GRP IS NOT NULL )								AS INCEVENT_FLAG
				,COALESCE(A.SUBGROUP,B.SUBGROUP)					AS SUBGROUP
				,COALESCE(A.ENCTYPE_A,B.ENCTYPE_B)					AS ENCTYPE
				,UPCASE(COALESCE(A.PDX_A,B.PDX_B)) 				AS PDX
				,A.WASHTYP
				,A.WASHPER
				,A.MIN_DAYS_FR_INDEX_FOR_COMP_WND
				,A.MAX_DAYS_FR_INDEX_FOR_COMP_WND
				,A.MIN_DAYS_FR_INDEX_FOR_RISK_WND
				,A.MAX_DAYS_FR_INDEX_FOR_RISK_WND
							
				
	FROM 	(	SELECT  A1.*,LKP.CODE_VALUE_C AS ENCTYPE_A, LKP2.CODE_VALUE_C AS PDX_A
					FROM	&EVENT. 							A1
			RIGHT JOIN	&LKP_FOR_ALL_CODES.	(WHERE=(COLUMN_NAME="ENCTYPE"))		    LKP 
			 		ON LKP.CODE_VALUE_C=A1.ENCTYPE OR A1.ENCTYPE IS NULL
			RIGHT JOIN	&LKP_FOR_ALL_CODES.	(WHERE=(COLUMN_NAME="PDX"))		LKP2 
			  	ON	LKP2.CODE_VALUE_C=A1.PDX OR A1.PDX IS NULL
				)	A
	FULL JOIN(SELECT 	B1.*,LKP.CODE_VALUE_C AS ENCTYPE_B, LKP2.CODE_VALUE_C AS PDX_B
					 FROM	&INCEVENT. 							B1
			RIGHT JOIN	&LKP_FOR_ALL_CODES.	(WHERE=(COLUMN_NAME="ENCTYPE"))		    LKP 
			 		ON LKP.CODE_VALUE_C=B1.ENCTYPE OR B1.ENCTYPE IS NULL
			RIGHT JOIN	&LKP_FOR_ALL_CODES.	(WHERE=(COLUMN_NAME="PDX"))		LKP2 
			  	ON	LKP2.CODE_VALUE_C=B1.PDX OR B1.PDX IS NULL
				) B
		ON A.EVENT_GRP=B.EVENT_GRP
	 AND  A.SUBGROUP=B.SUBGROUP 
	 AND A.ENCTYPE_A=B.ENCTYPE_B
	 AND A.PDX_A=B.PDX_B
	
		WHERE	COALESCE(A.EVENT_GRP,B.EVENT_GRP) 	 IS NOT NULL
		AND 		( 	CALCULATED PDX="P" AND CALCULATED ENCTYPE IN ("IP", "IS" )
					 OR CALCULATED PDX IS NULL
					 )
	    ORDER BY  EVENT_GRP
						,EVENT_FLAG
						,INCEVENT_FLAG
						,SUBGROUP
						,ENCTYPE
						,PDX
	;
QUIT;

* Find min and max exposure periods dates for all study groups;

PROC SQL;
 CREATE TABLE &tmp.._EXPOSURE_PERIOD_LIMITS AS	
 	SELECT MIN(EXPOS_PERIOD_STRT_DT) 	FORMAT=DATE9.  	AS MIN_EXPOS_PERIOD_STRT_DT 
				,MAX(EXPOS_PERIOD_END_DT) 	FORMAT=DATE9.	AS MAX_EXPOS_PERIOD_END_DT 
			FROM &tmp.._EXPOSURE_PERIODS
	;
QUIT;

* Find max WASHPER for all study groups for QUERY conditions;

PROC SQL;
 CREATE TABLE &tmp.._WASHPER_MAX AS	
 	SELECT MAX(WASHPER) 				AS MAX_WASHPER 
				,MAX(WASHTYP="MIN") 		AS WASHTYP_EQUAL_MIN
			FROM &QUERY.
	;
QUIT;

 * Find max WASHPER for all study groups for EVENT conditions;


PROC SQL;
 CREATE TABLE &tmp.._EVENT_DAYS_LIMITS AS	
 	SELECT MAX(WASHPER) 				AS MAX_EVENT_WASHPER
				,MAX(WASHTYP="MIN") 		AS EVENT_WASHTYP_EQUAL_MIN
,MIN(MIN(MIN_DAYS_FR_INDEX_FOR_COMP_WND,MIN_DAYS_FR_INDEX_FOR_RISK_WND)) AS MIN_DAYS_EVENT_WIND
,MAX(MAX(MAX_DAYS_FR_INDEX_FOR_COMP_WND,MAX_DAYS_FR_INDEX_FOR_RISK_WND)) AS MAX_DAYS_EVENT_WIND
			FROM &EVENT.
	;
QUIT;

PROC SQL;
 CREATE TABLE &tmp.._EVENT_DAYS_LIMITS_BY_GRP AS	
 	SELECT  EVENT_GRP
				,MAX(WASHPER) 				AS WASHPER
				,MAX(WASHTYP="MIN") 		AS EVENT_WASHTYP_EQUAL_MIN
,MIN(MIN(MIN_DAYS_FR_INDEX_FOR_COMP_WND,MIN_DAYS_FR_INDEX_FOR_RISK_WND)) AS MIN_DAYS_EVENT_WIND
,MAX(MAX(MAX_DAYS_FR_INDEX_FOR_COMP_WND,MAX_DAYS_FR_INDEX_FOR_RISK_WND)) AS MAX_DAYS_EVENT_WIND
			FROM &EVENT.
			 GROUP BY EVENT_GRP
	;
QUIT;


* Combine all dates to find min and max dates for initial extract of data from MSCDM tables;

DATA &tmp.._QUERY_EVENT_DATES_LIMITS;

			  SET &tmp.._EXPOSURE_PERIOD_LIMITS;
			  SET &tmp.._WASHPER_MAX;
			  SET &tmp.._EVENT_DAYS_LIMITS;
  		;
    IF MISSING(&QUERYFROM.)=1 THEN QUERYFROM="01JAN1900"d;
	ELSE QUERYFROM=&QUERYFROM.;

	IF WASHTYP_EQUAL_MIN 
	 THEN 	MIN_QUERY_DATE=QUERYFROM;
    ELSE 	MIN_QUERY_DATE=MIN_EXPOS_PERIOD_STRT_DT-MAX_WASHPER;

	MAX_QUERY_DATE=MAX_EXPOS_PERIOD_END_DT;

IF EVENT_WASHTYP_EQUAL_MIN
  THEN 	MIN_EVENT_DATE=QUERYFROM;
ELSE 	MIN_EVENT_DATE=MIN_EXPOS_PERIOD_STRT_DT+SUM(-MAX_EVENT_WASHPER,MIN_DAYS_EVENT_WIND);

	MAX_EVENT_DATE=MAX_EXPOS_PERIOD_END_DT +MAX_DAYS_EVENT_WIND;

	MIN_QUERY_EVENT_DT=MIN(MIN_QUERY_DATE,MIN_EVENT_DATE);
	MAX_QUERY_EVENT_DT=MAX(MAX_QUERY_DATE,MAX_EVENT_DATE);

	FORMAT QUERYFROM MIN_QUERY_DATE MAX_QUERY_DATE MIN_EVENT_DATE MAX_EVENT_DATE 
				 MIN_QUERY_EVENT_DT MAX_QUERY_EVENT_DT	DATE9. ;


		CALL SYMPUT("min_query_date",COMPRESS(PUT(MIN_QUERY_DATE,8.)) );
		CALL SYMPUT("max_query_date",COMPRESS(PUT(MAX_QUERY_DATE,8.)) );

		CALL SYMPUT("min_event_date",COMPRESS(PUT(MIN_EVENT_DATE,8.)) );
		CALL SYMPUT("max_event_date",COMPRESS(PUT(MAX_EVENT_DATE,8.)) );

		CALL SYMPUT("min_query_event_dt",COMPRESS(PUT(MIN_QUERY_EVENT_DT,8.)) );
		CALL SYMPUT("max_query_event_dt",COMPRESS(PUT(MAX_QUERY_EVENT_DT,8.)) );
 PUT _ALL_;
RUN;

			* Create tables with distinct set of SUBGROUP codes. ;

   PROC SORT DATA=&QUERY.(KEEP=SUBGROUP) OUT=&tmp.._QUERY_SUBGR NODUPKEY;
	 BY SUBGROUP;
   RUN;

   
   PROC SORT DATA=&INCQUERY.(KEEP=SUBGROUP) OUT=&tmp.._INCQUERY_SUBGR NODUPKEY;
	 BY SUBGROUP;
   RUN;

   PROC SORT DATA=&EVENT.(KEEP=SUBGROUP) OUT=&tmp.._EVENT_SUBGR NODUPKEY;
	 BY SUBGROUP;
   RUN;

   
   PROC SORT DATA=&INCEVENT.(KEEP=SUBGROUP) OUT=&tmp.._INCEVENT_SUBGR NODUPKEY;
	 BY SUBGROUP ;
   RUN;

* Combine tables with distinct set of SUBGROUP codes into a single table.
* Create flags that identify source (role) of a SUBGROUP ;
* The will be used for initial extract from the main MSCDM tables;

   DATA &tmp.._ALL_SUBGR;
    MERGE &tmp.._QUERY_SUBGR(IN=QUERY_TBL_FLAG0) 
				&tmp.._INCQUERY_SUBGR(IN=INCQUERY_TBL_FLAG0) 
				&tmp.._EVENT_SUBGR(IN=EVENT_TBL_FLAG0)
				&tmp.._INCEVENT_SUBGR(IN=INCEVENT_TBL_FLAG0)
				;
		 BY SUBGROUP ;

					QUERY_TBL_FLAG=QUERY_TBL_FLAG0;
					INCQUERY_TBL_FLAG=INCQUERY_TBL_FLAG0;
					EVENT_TBL_FLAG=EVENT_TBL_FLAG0;
					INCEVENT_TBL_FLAG=INCEVENT_TBL_FLAG0;
	RUN;

	
  PROC SORT DATA=&QUERY_CODES. OUT=&tmp.._QUERY_CODES NODUPKEY;
   BY SUBGROUP  CODETYPE  CODE ;
   RUN;
 
   
  PROC SORT DATA=&EVENT_CODES. OUT=&tmp.._EVENT_CODES NODUPKEY;
   BY SUBGROUP  CODETYPE  CODE ;
   RUN;

    * Combine codes used to define query conditions with codes used to define event conditions.
    * This is done to minimize the number of times one needs to query main MSCDM tables;
    * The input tables has all type of codes (NDC, PX, DX) combined together. 
    * The step below separates them into code tables for NDCs, diagnosis and procedures.  
    * The latter tables which will be used for initial extract from the main MSCDM tables.;
    * The code below also checks for the presence of wild cards in the codes and creates
    * corresponding flag for latter use;

   
   DATA &tmp.._NDC
			 &tmp.._DIAG
			 &tmp.._PROC
			;
    MERGE &tmp.._QUERY_CODES &tmp.._EVENT_CODES ;
	 BY SUBGROUP  CODETYPE  CODE ;

	 *Translate wild card characters to the standard SQL wild characters ;

	  CODE=TRANSLATE(CODE, '%' , "*" , "_","?");
	  * Remove period if it exists. Some sites have period in DX and PX codes and some do not ;
	  CODE=COMPRESS(CODE,".");

	  CODE_LENGTH=LENGTH(CODE);

	  
	  WILD_CARD_FLAG=(INDEXC(CODE, '%' , "_") >0);

   IF 	CODETYPE ="RX09"  AND WILD_CARD_FLAG=0 THEN
   										DO;
							WILD_CARD_FLAG=1;
							CODE=TRIM(CODE) || '%' ;
										END;
IF CODETYPE IN:("DX") THEN DO; CODETYPE=COMPRESS(CODETYPE,"DX"); OUTPUT &tmp.._DIAG; 	END;
IF CODETYPE IN:("PX") THEN DO; CODETYPE=COMPRESS(CODETYPE,"PX"); 	OUTPUT &tmp.._PROC; 	END; 
IF CODETYPE IN:("RX") THEN DO; CODETYPE=COMPRESS(CODETYPE,"RX"); OUTPUT &tmp.._NDC; 	END; 

  		KEEP SUBGROUP CODETYPE CODE CODE_LENGTH WILD_CARD_FLAG ;   
   RUN;

   
   TITLE "NDC SUBGROUP frequencies for input table";
  PROC FREQ DATA=&tmp.._NDC;
    TABLES SUBGROUP*WILD_CARD_FLAG /MISSING LIST NOCUM;
   RUN;

   TITLE "DX SUBGROUP frequencies  for input table";
  PROC FREQ DATA=&tmp.._DIAG;
    TABLES SUBGROUP*CODETYPE*WILD_CARD_FLAG /MISSING LIST NOCUM;
   RUN;

     TITLE "PX SUBGROUP frequencies  for input table";
   PROC FREQ DATA=&tmp.._PROC;
    TABLES SUBGROUP*CODETYPE*WILD_CARD_FLAG /MISSING LIST NOCUM;
   RUN;

* The steps below create tables with complete set of all NDC, PX, and DX codes ever used 
* by the MSCDM data warehouse at a given site;
* This will be used latter to create an explicit list of codes of interest without wild cards and
* in the format used by a site (for example, with or without a period in DX or PX codes);	

   PROC SQL;
     CREATE TABLE &tmp.._ALL_NDC AS  
	     SELECT DISTINCT NDC
	  		FROM INDATA.&DisTable.   
	  ;
   QUIT;

 PROC SQL;
  CREATE TABLE &tmp.._ALL_DX AS  
    SELECT DISTINCT DX_CODETYPE, DX
	  FROM INDATA.&DiaTable.
	  ;
 QUIT;

 PROC SQL;
  CREATE TABLE &tmp.._ALL_PX AS  
    SELECT DISTINCT PX_CODETYPE, PX
	  FROM INDATA.&PROCTABLE.
	  ;
 QUIT;

  TITLE " PX_CODETYPE frequencies  for all PX ever used at a site";
   PROC FREQ DATA=&tmp.._ALL_PX;
    TABLES PX_CODETYPE /MISSING;
   RUN;

   TITLE;
 
   * The query gives warning message to the log about Cartesian product.;
	* This is intentional. Ignore it. ;

 /*  %chk_drop_table(tbl_name=&tmp2.._NDCS_&out_tbl_prefix.);*/

  PROC SQL;
  CREATE TABLE &tmp2.._NDC_&out_tbl_prefix. AS  
    SELECT    	 DISTINCT
					A.SUBGROUP
					,B.NDC
	  FROM 	 &tmp.._NDC			A
	  			,&tmp.._ALL_NDC  B 
			WHERE (WILD_CARD_FLAG =0 AND B.NDC=A.CODE)
				   OR (WILD_CARD_FLAG =1 AND B.NDC LIKE TRIM(A.CODE))
			ORDER BY A.SUBGROUP, B.NDC
	  ;
 QUIT;

 %LET ndc_codes_numb=&SQLOBS;
 %PUT ndc_codes_numb=&ndc_codes_numb.;

 /*%chk_drop_table(tbl_name=&tmp2.._DX_&out_tbl_prefix.);*/
 
  PROC SQL;
  CREATE TABLE  &tmp2.._DX_&out_tbl_prefix. AS  
    SELECT  DISTINCT
				 A.SUBGROUP
				,B.DX_CODETYPE 
				,B.DX
	  FROM 	 &tmp.._DIAG	 A
	  			,&tmp.._ALL_DX  B 
			
 WHERE A.CODETYPE=B.DX_CODETYPE
	 AND	(	  (WILD_CARD_FLAG =0 AND COMPRESS(B.DX,".")=A.CODE )
	   		 OR (WILD_CARD_FLAG =1 AND TRIM(COMPRESS(B.DX,".")) LIKE TRIM(A.CODE) )
			)
			ORDER BY  A.SUBGROUP
						,B.DX_CODETYPE
							,B.DX
	  ;
 QUIT;


 %LET dx_codes_numb=&SQLOBS;
 %PUT dx_codes_numb=&dx_codes_numb.;

  /*%chk_drop_table(tbl_name=&tmp2.._PX_&out_tbl_prefix.);*/

  PROC SQL;
  CREATE TABLE  &tmp2.._PX_&out_tbl_prefix. AS  
    SELECT  DISTINCT
				 A.SUBGROUP
				,B.PX_CODETYPE 
				,B.PX
	  FROM 	 &tmp.._PROC	 A
	  			,&tmp.._ALL_PX  B 
			
 WHERE A.CODETYPE=B.PX_CODETYPE
	 AND	(	  (WILD_CARD_FLAG =0 AND COMPRESS(B.PX,".")=A.CODE )
	   		 OR (WILD_CARD_FLAG =1 AND TRIM(COMPRESS(B.PX,".")) LIKE TRIM(A.CODE) )
			)
			ORDER BY  A.SUBGROUP
							,B.PX_CODETYPE
							,B.PX
	  ;
 QUIT;

%LET px_codes_numb=&SQLOBS;
%PUT px_codes_numb=&px_codes_numb.;

%MACRO extract_rx_dx_px_data;

* Extract RX data;

%IF &ndc_codes_numb. > 0 %THEN
								%DO;
PROC SQL ;
CREATE TABLE &tmp.._RXEXTRACT(RENAME=(RxDate=ADATE NDC=CODE)) AS
SELECT  B.SUBGROUP
			,A.*
			," "				LENGTH=2		AS ENCTYPE
			,"RX11"			LENGTH=4		AS CODETYPE
			," " 				LENGTH=1		AS PDX

   FROM  INDATA.&DISTABLE.   		A,
   			 &tmp2.._NDC_&out_tbl_prefix.	B
		WHERE A.NDC=B.NDC
			AND A.RxDate BETWEEN &min_query_event_dt. AND &max_query_event_dt.
			AND RXSUP>0 /* Defensive coding. Added 06/11/2013 */
		ORDER BY B.SUBGROUP, A.PatID, A.RxDate, A.NDC
		;
 QUIT;
	

 /*%chk_drop_table(tbl_name=&tmp2.._NDCS_&out_tbl_prefix.);*/

TITLE "Distributions for RX data extract." ;

PROC FREQ DATA=&tmp.._RXEXTRACT;
  TABLES SUBGROUP CODETYPE ADATE ENCTYPE  /MISSING LIST;
   FORMAT ADATE MONYY7.;
RUN;


PROC MEANS DATA=&tmp.._RXEXTRACT MIN P25 MEDIAN P75 MAX;
 CLASS SUBGROUP;
  VAR RxSup RxAmt  ;
RUN;	

%LET ndc_extract=&tmp.._RXEXTRACT;
				%END;
%ELSE %LET ndc_extract=%STR();
 
 * Extract DX data;

%IF &dx_codes_numb. > 0 %THEN
				  %DO;

PROC SQL ;
CREATE TABLE &tmp.._DIAGEXTRACT(DROP=DX DX_CODETYPE) AS
SELECT  B.SUBGROUP
			,A.*
			,A.DX_CODETYPE 	LENGTH=4		AS CODETYPE
			,A.DX					LENGTH=11  AS CODE
			,0							LENGTH=4		AS RxSup
          	,0							LENGTH=4		AS RxAmt

   FROM  INDATA.&DiaTABLE.   		A ,
   			 &tmp2.._DX_&out_tbl_prefix. 	B
		WHERE A.DX=B.DX
			AND A.DX_CODETYPE=B.DX_CODETYPE
			AND A.ADATE BETWEEN &min_query_event_dt. AND &max_query_event_dt.
		ORDER BY B.SUBGROUP, PatID, ADATE, A.ENCTYPE, CODETYPE, CODE, PDX
   ;
QUIT;

/*%chk_drop_table(tbl_name=&tmp2.._DX_&out_tbl_prefix.);*/

TITLE "Distributions for DX data extract." ;

PROC FREQ DATA=&tmp.._DIAGEXTRACT;
  TABLES  SUBGROUP CODETYPE ADATE ENCTYPE  /MISSING LIST;
   FORMAT ADATE MONYY7.;
RUN;

%LET dx_extract=&tmp.._DIAGEXTRACT;
				%END;

%ELSE %LET dx_extract=%STR();

* Extract PX data ;

%IF &px_codes_numb. > 0 %THEN
								%DO;
PROC SQL ;
CREATE TABLE  &tmp.._PROCEXTRACT(DROP=PX PX_CODETYPE)  AS
SELECT  B.SUBGROUP
			,A.*
			,A.PX_CODETYPE 	LENGTH=4		AS CODETYPE
			,A.PX						LENGTH=11  AS CODE
			,1							LENGTH=4		AS RxSup
          	,0							LENGTH=4		AS RxAmt
			," " 											AS PDX

   FROM  INDATA.&PROCTABLE.   A,
   			 &tmp2.._PX_&out_tbl_prefix. B
		WHERE A.PX=B.PX
			AND A.PX_CODETYPE=B.PX_CODETYPE
			AND A.ADATE BETWEEN &min_query_event_dt. AND &max_query_event_dt.
		ORDER BY B.SUBGROUP, A.PatID, A.ADATE, A.ENCTYPE, CODETYPE, CODE
   ;
QUIT;

 /*%chk_drop_table(tbl_name=&tmp2.._PX_&out_tbl_prefix.);*/

TITLE "Distributions for PX data extract." ;

PROC FREQ DATA=&tmp.._PROCEXTRACT;
  TABLES  SUBGROUP CODETYPE ADATE ENCTYPE  /MISSING LIST;
   FORMAT ADATE MONYY7.;
RUN;

%LET px_extract=&tmp.._PROCEXTRACT;
				%END;

%ELSE %LET px_extract=%STR();

* Combine NDC, DX and PX data extracts.;
* Summarize RX data for dispenses on the same date; 
* Keep one record per combination of MSCDM variables that suppose to be unique per MSCDM data model;
* The last two steps are added in case a site does not completely comply with MSCDM data model;

DATA &tmp.._ALL_EXTRACTS(SORTEDBY=SUBGROUP PatID ADATE ENCTYPE CODETYPE CODE PDX
												RENAME=(sum_RxSup=RxSup sum_RxAmt=RxAmt)
											) 
				;
		  SET 	&ndc_extract. 
		  			&dx_extract.
					&px_extract.
				;
				BY SUBGROUP PatID ADATE ENCTYPE CODETYPE CODE PDX;

				 RETAIN sum_RxSup sum_RxAmt;

				IF FIRST.PDX THEN 
							DO;
				sum_RxSup=0;
				sum_RxAmt=0; 
							END;

				sum_RxSup=sum_RxSup+RxSup;
				sum_RxAmt=sum_RxAmt+RxAmt; 

			IF LAST.PDX;	

			DROP RxSup RxAmt;
RUN;

%MEND;

%extract_rx_dx_px_data;

* Separate extracted encounters into ones that will be used for population selection (query conditions);
* and the ones that will be used for adverse outcome (events conditions);
* Reset PDX to missing when it is not principal diagnosis because we need to treat missing and secondary;
* diagnosis in the same way;


DATA 	&tmp.._ALL_QUERY_INCQUERY_EXTRCT(DROP=EVENT_TBL_FLAG INCEVENT_TBL_FLAG
																		SORTEDBY=SUBGROUP PatID ADATE ENCTYPE CODETYPE CODE PDX
																		)  
			&tmp.._ALL_EVENTS_INCEVENTS_EXTRCT(DROP=QUERY_TBL_FLAG  INCQUERY_TBL_FLAG
																			SORTEDBY=SUBGROUP PatID ADATE ENCTYPE CODETYPE CODE PDX
																		) 
				;
		  		SET 	&tmp.._ALL_EXTRACTS;

		PDX=UPCASE(PDX);

		IF PDX NE "P" THEN PDX=" ";

			IF _N_=1 THEN 
						DO;
	 IF 0 THEN SET &tmp.._ALL_SUBGR(KEEP=SUBGROUP QUERY_TBL_FLAG INCQUERY_TBL_FLAG EVENT_TBL_FLAG INCEVENT_TBL_FLAG);
	DECLARE HASH h1(HASHEXP:8, DATASET:"&tmp.._ALL_SUBGR(KEEP=SUBGROUP QUERY_TBL_FLAG INCQUERY_TBL_FLAG EVENT_TBL_FLAG INCEVENT_TBL_FLAG)", DUPLICATE: "E" );

		h1.DEFINEKEY("SUBGROUP");
		h1.DEFINEDATA("QUERY_TBL_FLAG", "INCQUERY_TBL_FLAG", "EVENT_TBL_FLAG", "INCEVENT_TBL_FLAG");
		h1.DEFINEDONE();
						END;

			rc=h1.FIND();
 	IF rc NE 0  THEN CALL MISSING(QUERY_TBL_FLAG, INCQUERY_TBL_FLAG, EVENT_TBL_FLAG, INCEVENT_TBL_FLAG);

	IF      (QUERY_TBL_FLAG OR INCQUERY_TBL_FLAG)
	  AND (&min_query_date. <= ADATE <= &max_query_date.)
	THEN OUTPUT &tmp.._ALL_QUERY_INCQUERY_EXTRCT;
 
	IF 			(EVENT_TBL_FLAG OR INCEVENT_TBL_FLAG)
		AND ( &min_event_date. <= ADATE <= &max_event_date. )
	THEN OUTPUT &tmp.._ALL_EVENTS_INCEVENTS_EXTRCT; 
RUN;

* Add population selection parameters from the mapping table to the "QUERY/ INQUERY" data extract ;
 
* The query gives warning message to the log about Cartesian product.;
* It is triggered by the absence of equal condition in the join to mapping table EXPOSURE_PERIODS;
* This is intentional. Ignore it. ;

PROC SQL ;
  CREATE TABLE &tmp.._ENC_QUERY_INCQUERY_PARMS AS
      SELECT B.QUERY_GRP
				, A.PatID
				, A.ADATE		
				, B.QUERY_FLAG
				, B.INCQUERY_FLAG
				, C.EXPOSURE_PERIOD_CD
				, A.ENCTYPE
				, A.SUBGROUP
				, A.CODETYPE
				, A.CODE
				, A.PDX
				, A.RXSUP
				, A.RXAMT
				, B.WASHTYP
				, B.WASHPER
				, B.EXCL_MEDS_ON_SAME_DAY
				, C.EXPOS_PERIOD_STRT_DT
				, C.EXPOS_PERIOD_END_DT

		FROM 		&tmp.._ALL_QUERY_INCQUERY_EXTRCT 		A
		INNER JOIN	&tmp.._QUERY_INCQUERY						B
				ON A.SUBGROUP=B.SUBGROUP
		LEFT JOIN &tmp.._EXPOSURE_PERIODS						C
				ON 	EXPOS_PERIOD_STRT_DT <= A.ADATE <=EXPOS_PERIOD_END_DT
		ORDER BY   QUERY_GRP
						, A.PatID
						, A.ADATE	
						, QUERY_FLAG
						, INCQUERY_FLAG	DESC
						, A.ENCTYPE
						, SUBGROUP
						, A.CODETYPE
						, A.CODE
						, A.PDX
			;
	QUIT;

* Apply population selection criteria to find patients with exposure to the drugs;
* or other encounters of interest;
  
DATA &tmp.._PAT_EXPOSURE_EPISODES
		 &tmp.._PAT_EXPOSURE_EPISODES_EXCL
			;
 	SET &tmp.._ENC_QUERY_INCQUERY_PARMS;
  	  BY QUERY_GRP PatID ADATE QUERY_FLAG DESCENDING INCQUERY_FLAG ;

   RETAIN PREV_ADATE2 EPISODE_NUMB  ;

   FORMAT PREV_ADATE2 DATE9. ;

   PREV_ADATE=LAG(ADATE);
    FST_PAT_ID=FIRST.PatID;
	LST_PAT_ID=LAST.PatID;

   IF FIRST.PatID THEN 
   						DO;
				PREV_ADATE=.;
				PREV_ADATE2=.;
				DAYS_TO_PREV_ADATE=.;
				DAYS_TO_PREV_ADATE2=.;
				WASHPER_FLAG=.;
				SAME_DAY_FLAG=.;
				EPISODE_NUMB=.;
				EXPOSURE_DT=.;
				EXPOSURE_FLAG=.;
						END;
	ELSE 			
						DO;
			* Note that PREV_ADATE is the date of a previous record and can be the same as ADATE.
			* The PREV_ADATE2 is date of the earliest incidence at previous date. 
			* By construction, it cannot be the same as  ADATE. ;
			* It is set and retained later in this data step;

			DAYS_TO_PREV_ADATE= ADATE -PREV_ADATE;

			SAME_DAY_FLAG=(DAYS_TO_PREV_ADATE=0);

			*06/10/2013. Added line below to accommodate case EXCL_MEDS_ON_SAME_DAY=0;
		IF PREV_ADATE2 NE . THEN 	DAYS_TO_PREV_ADATE2= ADATE -PREV_ADATE2;

		*     			We set WASHPER_FLAG=1 when:
		* a) There is previous incidence in a look back period from previous day to WASHPER days.;
		* b) If parameter EXCL_MEDS_ON_SAME_DAY=1 (from QUERY table) there is another
			    incidence on the same day;

			IF 	QUERY_FLAG THEN		
											DO;
		* 06/10/2013. Replaced DAYS_TO_PREV_ADATE with AYS_TO_PREV_ADATE2  to accommodate case EXCL_MEDS_ON_SAME_DAY=0;
		WASHPER_FLAG=(		 (1 <= DAYS_TO_PREV_ADATE2 <= WASHPER)
									OR (DAYS_TO_PREV_ADATE=0 AND EXCL_MEDS_ON_SAME_DAY)
									);
											END;	
		ELSE	
											DO; 
							EPISODE_NUMB=.; 
							EXPOSURE_DT=.; 
							WASHPER_FLAG=.;
											END;
						END;

    EXPOS_PERIOD_FLAG=(EXPOS_PERIOD_STRT_DT <= ADATE<= EXPOS_PERIOD_END_DT);
    
	LST_QUERY_FLAG=(LAST.QUERY_FLAG AND QUERY_FLAG);
	

   IF     		(WASHPER_FLAG=. OR WASHPER_FLAG=0)  
		AND  EXPOS_PERIOD_FLAG=1
		AND  LST_QUERY_FLAG=1 THEN
					DO;
		EPISODE_NUMB=SUM(EPISODE_NUMB,1); 
		EXPOSURE_DT=ADATE; 
		EXPOSURE_FLAG=1;
					END;
ELSE EXPOSURE_FLAG=0;

					*Changed line for WASHTYP IN ("MIN") after QC. 06/10/203 ;
	
IF 			WASHTYP IN ("MIN") AND EXPOS_PERIOD_FLAG AND FIRST.PatID AND QUERY_FLAG   
	THEN OUTPUT &tmp.._PAT_EXPOSURE_EPISODES;

ELSE IF 	WASHTYP IN ( "SING") AND EXPOSURE_FLAG AND  EPISODE_NUMB=1 
	THEN OUTPUT &tmp.._PAT_EXPOSURE_EPISODES;

ELSE IF WASHTYP IN ("MULT") AND EXPOSURE_FLAG  
 	THEN OUTPUT &tmp.._PAT_EXPOSURE_EPISODES;

ELSE OUTPUT &tmp.._PAT_EXPOSURE_EPISODES_EXCL;
		FORMAT PREV_ADATE EXPOSURE_DT DATE9.;

		* 06/10/2013. Added the line below to accommodate case EXCL_MEDS_ON_SAME_DAY=0;
		* Set PREV_ADATE2 from the last record for the same date. The variable is retained ;

		IF LAST.ADATE THEN PREV_ADATE2=ADATE;
		ELSE; * do nothing ;

 RUN;

TITLE "Check distributions for parameters used for population selection. Included patients";

 PROC FREQ DATA=&tmp.._PAT_EXPOSURE_EPISODES;
  TABLES   EXPOSURE_FLAG*EXPOSURE_DT
				EXPOSURE_FLAG*WASHPER_FLAG*EXPOS_PERIOD_FLAG*LST_QUERY_FLAG*SAME_DAY_FLAG 
				/MISSING LIST NOCUM;
  TABLES SAME_DAY_FLAG*WASHPER_FLAG*EXCL_MEDS_ON_SAME_DAY/MISSING LIST;
  * TABLES EPISODE_NUMB*WASHPER_FLAG*EXPOS_PERIOD_FLAG*EPISODE_INTERAPTED_FLAG /MISSING LIST NOCUM;
	FORMAT EXPOSURE_DT MONYY7.;
 RUN;

 PROC MEANS DATA=&tmp.._PAT_EXPOSURE_EPISODES N NMISS MIN MAX;
  VAR DAYS_TO_PREV_ADATE DAYS_TO_PREV_ADATE2;
 RUN;

 TITLE "Check distributions for parameters used for population selection. Excluded patients";

 PROC FREQ DATA=&tmp.._PAT_EXPOSURE_EPISODES_EXCL;
  TABLES   EXPOSURE_FLAG EXPOSURE_DT
				WASHPER_FLAG*EXPOS_PERIOD_FLAG*LST_QUERY_FLAG*SAME_DAY_FLAG 
				/MISSING LIST NOCUM;
  * TABLES EPISODE_NUMB*WASHPER_FLAG*EXPOS_PERIOD_FLAG*EPISODE_INTERAPTED_FLAG /MISSING LIST NOCUM;
	FORMAT EXPOSURE_DT MONYY7.;
 RUN;

 
 PROC MEANS DATA=&tmp.._PAT_EXPOSURE_EPISODES_EXCL N NMISS MIN MAX;
 CLASS WASHPER_FLAG;
  VAR DAYS_TO_PREV_ADATE DAYS_TO_PREV_ADATE2;
 RUN;

 * Create table with distinct list of patients that had exposure episode. ;
 * The list will be used to pull enrollment and demographics;

 /*%chk_drop_table(tbl_name=&tmp2.._PAT_&out_tbl_prefix.);*/

PROC SQL;
 CREATE TABLE &tmp2.._PAT_&out_tbl_prefix. AS 
 	SELECT DISTINCT PatID
		FROM &tmp.._PAT_EXPOSURE_EPISODES
	;
QUIT;

* Get all encounters for the population of interest;

PROC SQL;
 CREATE TABLE &tmp.._ENC_IP AS
   SELECT DISTINCT A.PatId, A.Adate, A.Ddate, A.EncType
     	FROM 			INDATA.&EncTable.    							A
		 INNER JOIN  &tmp2.._PAT_&out_tbl_prefix. B
					  ON	A.PatId=B.PatId
					AND A.EncType='IP'
		  ORDER BY A.PatId, A.Adate, A.Ddate
			;
	QUIT;

* Extract enrollment and demographics information for the population of interest;
* The ENR.ENR_START, ENR.ENR_END should not be NULL according to MSCDM but;
*  we add the corresponding conditions below just in case the site is not compliant.
;

	* 06/10/2013. In response to QC, added &ENROLGAP parameter to the condition for ENR.ENR_END;
	* and condition for ENR.ENR_START in the query below. These changes relax conditions for initial selection 
	* of enrollment periods. This is necessary to allow for enrollment periods  that can  satisfy
	* conditions for  ENR_START and ENR_END after macro %ms_episoderec  bridge enrollment periods 
	* with gap less than &ENROLGAP;
	

PROC SQL NOPRINT;
	CREATE TABLE &tmp.._PAT_ENR_DEM AS
		SELECT DEM.PatID,
       				DEM.BIRTH_DATE,
       				DEM.SEX,
       				ENR.ENR_START,
       				MIN(ENR.ENR_END,&max_query_event_dt.) AS ENR_END,
					ENR.DRUGCOV,
					ENR.MEDCOV
     		FROM 			INDATA.&DEMTABLE. 				AS DEM
			INNER JOIN   &&tmp2.._PAT_&out_tbl_prefix. 	AS PAT
   				ON DEM.PATID=PAT.PATID
			AND DEM.BIRTH_DATE IS NOT NULL 
    		INNER JOIN      INDATA.&ENRTABLE. 			AS ENR
   				ON DEM.PATID=ENR.PATID
			AND (ENR.ENR_END >= &min_query_event_dt. -&ENROLGAP. OR ENR.ENR_END IS NULL)
			AND ENR.ENR_START IS NOT NULL
			AND ENR.ENR_START <=&max_query_event_dt. +&ENROLGAP.
   ORDER BY DEM.PatID, ENR.ENR_START, ENR.ENR_END   
				;
QUIT;

* Create enrollment spans which bridge enrollment intervals separated by allowed grace period;
* 06/10/2013. Replaced hard-coded value for ENROLGAP with parameter &ENROLGAP.;

%ms_episoderec(infile=&tmp.._PAT_ENR_DEM
							,outfile=&tmp.._PAT_DEM_ENR_SPANS
							,medcov=Y
							,drugcov=Y
							,enrstart=ENR_START
							,enrend=ENR_END
							,enrolgap=&ENROLGAP.
							,removedup=Y
							);

* Add enrollment and demographics data to the table with patients who had exposures;
* Calculate age in units (years, months, etc) defined by parameter AGE_INTERVAL_TYPE;
* Exclude patients that do not have age in the required age intervals;

	*06/11/2013. Changed the date at which age is calculated 
	* from EXPOS_PERIOD_STRT_DT to EXPOSURE_DT.;


 PROC SQL;
   CREATE TABLE &tmp.._PAT_EXP_EP_DEM_ENR AS
   	SELECT   A.QUERY_GRP
				 ,A.EXPOS_PERIOD_STRT_DT
				, A.PatID
				, A.EXPOSURE_DT
				, A.SUBGROUP
				, A.WASHTYP
				, A.WASHPER
				, D.AGE_INTERVAL_TYPE
				, D.AGE_GROUP
				, D.AGE_MIN
				, D.AGE_MAX
				, D.AGE_GRP_DSC
				, (CASE 
					WHEN  BIRTH_DATE IS NULL
					THEN .
					ELSE	INTCK("MONTH", BIRTH_DATE, EXPOSURE_DT, "C")/12
				    END) AS AGE_YM
				, (CASE 
					WHEN AGE_INTERVAL_TYPE IS NULL OR BIRTH_DATE IS NULL
					THEN .
					ELSE	INTCK(AGE_INTERVAL_TYPE, BIRTH_DATE, EXPOSURE_DT, "C")
				    END) AS AGE
				, B.EStart 					AS ENR_START
				, B.EEnd 					AS ENR_END
				, B.BIRTH_DATE
				, B.SEX
				  ,(A.EXPOSURE_DT-  B.EStart) 	AS  DAYS_ENR_BEFORE_EXPOS
				  ,(B.EEnd-A.EXPOSURE_DT) 		AS  DAYS_ENR_AFTER_EXPOS	
				, A.EXPOSURE_PERIOD_CD	
				, A.EXPOS_PERIOD_END_DT

  		FROM 			&tmp.._PAT_EXPOSURE_EPISODES		A
		LEFT JOIN  	&tmp.._PAT_DEM_ENR_SPANS			B
					 ON 	A.PatID=B.PatID
					AND A.EXPOSURE_DT BETWEEN B.EStart AND B.EEnd  
		LEFT JOIN 	&tmp.._AGE_GROUPS								D
					ON A.QUERY_GRP = D.QUERY_GRP	
		AND (CASE 
					WHEN AGE_INTERVAL_TYPE IS NULL OR BIRTH_DATE IS NULL
					THEN 0
			ELSE	(INTCK(AGE_INTERVAL_TYPE, BIRTH_DATE, EXPOSURE_DT, "C")
						 		BETWEEN  D.AGE_MIN AND D.AGE_MAX )
				 END)=1
			 ORDER BY  A.QUERY_GRP					
							, A.EXPOSURE_PERIOD_CD	
							, A.PatID
							, A.EXPOSURE_DT 
							, ENR_START
			;
	QUIT;

	* Select patients who were enrolled withing WASHPER for population selection;

	DATA &tmp.._PAT_EXP_EP_DEM_ENR_INCL(SORTEDBY=QUERY_GRP EXPOS_PERIOD_STRT_DT PatID EXPOSURE_DT)
			  &tmp.._PAT_EXP_EP_DEM_ENR_EXCL(SORTEDBY=QUERY_GRP EXPOS_PERIOD_STRT_DT PatID EXPOSURE_DT)
				;
	  SET &tmp.._PAT_EXP_EP_DEM_ENR;
		BY QUERY_GRP EXPOS_PERIOD_STRT_DT PatID EXPOSURE_DT;

		AGE_CONDITION_FLAG= (AGE >=0 AND AGE_MIN <= AGE <= AGE_MAX );

		ENR_WITHIN_WASHPER_FLAG=(		    DAYS_ENR_BEFORE_EXPOS >=WASHPER
														AND DAYS_ENR_AFTER_EXPOS >=0
														);

	  IF (			ENR_WITHIN_WASHPER_FLAG 
		 	AND 	AGE_CONDITION_FLAG
			) 
	  	THEN		OUTPUT &tmp.._PAT_EXP_EP_DEM_ENR_INCL;
	  ELSE 		OUTPUT &tmp.._PAT_EXP_EP_DEM_ENR_EXCL;		
	RUN;

	
	TITLE "Distribution of various variables for patients who satisfy enrollment and age condition.";

	PROC MEANS DATA=&tmp.._PAT_EXP_EP_DEM_ENR_INCL MIN MAX;
	  BY QUERY_GRP;
	  CLASS  ENR_WITHIN_WASHPER_FLAG AGE_CONDITION_FLAG;
	   VAR AGE DAYS_ENR_BEFORE_EXPOS DAYS_ENR_AFTER_EXPOS;
	RUN;

	TITLE "Distribution of various variables for patients who do not satisfy enrollment and age condition.";

	PROC MEANS DATA=&tmp.._PAT_EXP_EP_DEM_ENR_EXCL MIN MAX;
	  BY QUERY_GRP;
	  CLASS  ENR_WITHIN_WASHPER_FLAG AGE_CONDITION_FLAG;
	   VAR AGE_YM;
	RUN;
	
	 * Record EncType to "IP" for encounters that occurs during inpatient stay;

PROC SQL;
 CREATE TABLE &tmp.._ALL_EVENTS_INCEVENTS_EXTRCT_1	 AS
   	SELECT DISTINCT 
				A.*
				,CASE
				  WHEN 		 A.ORIG_ENCTYPE IS NULL OR B.ENCTYPE IS NULL 					
					THEN A.ORIG_ENCTYPE
				  	ELSE B.ENCTYPE
				END										AS ENCTYPE
				,(B.ENCTYPE IS NOT NULL)		AS IP_PERIOD_FLAG

FROM 		&tmp.._ALL_EVENTS_INCEVENTS_EXTRCT(RENAME=(ENCTYPE=ORIG_ENCTYPE)) A
LEFT JOIN &tmp.._ENC_IP										   B
		 ON A.PatId = B.PatId 
	    AND B.ADate <= A.ADate <=  MAX(B.ADATE,B.DDATE)
			;
QUIT;

	TITLE " Crosstab of ORIG_ENCTYPE*ENCTYPE for events of interest";

	PROC FREQ DATA=&tmp.._ALL_EVENTS_INCEVENTS_EXTRCT_1;
	 TABLES IP_PERIOD_FLAG*ORIG_ENCTYPE*ENCTYPE/MISSING LIST;
	RUN;

	TITLE;

	* Find patients that can have outcomes of interest and add parameters for adverse event conditions;

 PROC SQL ;
   CREATE TABLE &tmp.._PAT_OUTCOMES_PARMS AS
   	SELECT  C.EVENT_GRP
				,C.EVENT_FLAG
				,C.INCEVENT_FLAG
				,B.SUBGROUP
				,B.PatID
				,B.ADATE
				,B.ENCTYPE
				,B.PDX 
				,C.WASHTYP
				,C.WASHPER
				,C.MIN_DAYS_FR_INDEX_FOR_COMP_WND
				,C.MAX_DAYS_FR_INDEX_FOR_COMP_WND
				,C.MIN_DAYS_FR_INDEX_FOR_RISK_WND
				,C.MAX_DAYS_FR_INDEX_FOR_RISK_WND
				
  		FROM 	(SELECT DISTINCT PatID 
							FROM &tmp.._PAT_EXP_EP_DEM_ENR_INCL
					)																				A
		INNER JOIN &tmp.._ALL_EVENTS_INCEVENTS_EXTRCT_1			B
					ON A.PatID=B.PatID
		INNER JOIN	&tmp.._EVENT_INCEVENT								C									
					ON  B.SUBGROUP=C.SUBGROUP
				  AND  B.ENCTYPE=C.ENCTYPE 
				  AND  B.PDX=C.PDX 
		
		 	ORDER BY 	  C.EVENT_GRP
							, B.PatID
							, B.ADATE	
							, EVENT_FLAG
							, INCEVENT_FLAG	DESC
			;
	QUIT;

	TITLE "Distribution of patients outcomes by EVENT_GRP before matching to corresponding QUERY_GRP. ";

 PROC FREQ DATA=&tmp.._PAT_OUTCOMES_PARMS;
  *BY EVENT_GRP;
  TABLES   EVENT_GRP*EVENT_FLAG*INCEVENT_FLAG*ENCTYPE ADATE
  EVENT_GRP*SUBGROUP*EVENT_FLAG*INCEVENT_FLAG
				/MISSING LIST NOCUM;
  * TABLES EPISODE_NUMB*WASHPER_FLAG*EXPOS_PERIOD_FLAG*EPISODE_INTERAPTED_FLAG /MISSING LIST NOCUM;
	FORMAT ADATE MONYY7.;
 RUN;



* Find patients who satisfy WASHPER condition for adverse events ;
  
DATA &tmp.._PAT_EVENT_EPISODES
		 &tmp.._PAT_EVENT_EPISODES_EXCL
			;
 	SET &tmp.._PAT_OUTCOMES_PARMS;
  	  BY EVENT_GRP PatID ADATE EVENT_FLAG DESCENDING INCEVENT_FLAG ;

   RETAIN PREV_ADATE EVENT_EPISODE_NUMB  ;

   *06/10/2013. In response to QC, changed the way the variable PREV_ADATE is defined. ;
   * Instead of the date from the previous record (which can be on the same date)
   * it is now defined as a date of a previous earliest event before the current date. ;
   * The variable is set at the end of the current data step and is retained;
    
   IF FIRST.PatID THEN 
   						DO;
				PREV_ADATE=.;
				DAYS_TO_PREV_ADATE=.;
				WASHPER_FLAG=.;
				EVENT_EPISODE_NUMB=.;
				EVENT_DT=.;
				OUTCOME_FLAG=.;
						END;
	ELSE 			
						DO;
		* 06/10/2013. Added IF/ ELSE statement  to avoid operation on missing warning in the log;
		* Set PREV_ADATE from the last record for the same date. The variable is retained ;

		IF PREV_ADATE NE . THEN 	DAYS_TO_PREV_ADATE= ADATE -PREV_ADATE;
		ELSE DAYS_TO_PREV_ADATE=.;

		IF 	EVENT_FLAG THEN		
											DO;
		WASHPER_FLAG= (1 < =DAYS_TO_PREV_ADATE <= WASHPER);
											END;	
		ELSE	
											DO; 
							EVENT_EPISODE_NUMB=.; 
							EVENT_DT=.; 
							WASHPER_FLAG=.;
											END;
						END;
    
	LST_EVENT_FLAG=(LAST.EVENT_FLAG AND EVENT_FLAG);
	

   IF     		(WASHPER_FLAG=. OR WASHPER_FLAG=0)  
		AND  LST_EVENT_FLAG=1 THEN
					DO;
		EVENT_EPISODE_NUMB=SUM(EVENT_EPISODE_NUMB,1); 
		EVENT_DT=ADATE; 
		OUTCOME_FLAG=1;
					END;
ELSE OUTCOME_FLAG=0;

IF 			WASHTYP IN ("MIN") AND OUTCOME_FLAG AND FIRST.PatID    
	THEN OUTPUT &tmp.._PAT_EVENT_EPISODES;

ELSE IF 	WASHTYP IN ( "SING") AND OUTCOME_FLAG AND  EVENT_EPISODE_NUMB=1 
	THEN OUTPUT &tmp.._PAT_EVENT_EPISODES;

ELSE IF WASHTYP IN ("MULT") AND OUTCOME_FLAG  
 	THEN OUTPUT &tmp.._PAT_EVENT_EPISODES;

ELSE OUTPUT &tmp.._PAT_EVENT_EPISODES_EXCL;

*06/10/2013. Set PREV_ADATE using the last record for the same date. The variable is retained ;

		IF LAST.ADATE THEN PREV_ADATE=ADATE;
		ELSE; * do nothing ;
	
		FORMAT PREV_ADATE EVENT_DT DATE9.;
 RUN;

 TITLE1 "Distributions for members who satisfy WASHPER condition." ;
 TITLE2 "WASHPER is missing for first encounter or EVENT_FLAG=0";
 TITLE3 "Before matching to corresponding QUERY_GRP. ";

 PROC MEANS DATA=&tmp.._PAT_EVENT_EPISODES N MIN MAX;
 CLASS EVENT_GRP;
  VAR DAYS_TO_PREV_ADATE;
  RUN;


  TITLE1 "Distributions for members who do not satisfy WASHPER condition." ;
 TITLE2 "Before matching to corresponding QUERY_GRP. ";

 PROC MEANS DATA=&tmp.._PAT_EVENT_EPISODES_EXCL N MIN MAX;
 CLASS EVENT_GRP WASHPER_FLAG LST_EVENT_FLAG/ MISSING;
  VAR DAYS_TO_PREV_ADATE;
  RUN;

TITLE1;
TITLE2;
TITLE3;

 * Combine exposure information and adverse outcome information into a single dataset ;
 * Match query conditions (QUERY_GRP) with the corresponding event conditions (EVEN_GRP);
 * Add study group code;

 PROC SQL;
  CREATE TABLE &tmp.._PAT_EXPOS_OUTCOMES AS
  			SELECT
  				  G.GROUP
  				, G.QUERY_GRP
				, G.EVENT_GRP
				, A.EXPOSURE_PERIOD_CD
				, A.EXPOS_PERIOD_STRT_DT		
				, A.EXPOS_PERIOD_END_DT
				, A.PatID
				, A.EXPOSURE_DT
				, A.AGE_INTERVAL_TYPE
				, A.AGE_GROUP
				, A.AGE_GRP_DSC			
				, A.SEX
				, A.DAYS_ENR_AFTER_EXPOS
				, B.EVENT_DT
				,(B.EVENT_DT-A.EXPOSURE_DT) 				AS DAYS_FROM_EXPOS_TO_EVENT
				,E.MAX_DAYS_EVENT_WIND
				,B.MIN_DAYS_FR_INDEX_FOR_COMP_WND 
				,B.MAX_DAYS_FR_INDEX_FOR_COMP_WND
				,B.MIN_DAYS_FR_INDEX_FOR_RISK_WND
				,B.MAX_DAYS_FR_INDEX_FOR_RISK_WND
		
		FROM 		 	&GROUPS_MAPPING.								G
		INNER JOIN	&tmp.._EVENT_DAYS_LIMITS_BY_GRP		E
					ON	G.EVENT_GRP=E.EVENT_GRP
    	INNER JOIN 	&tmp.._PAT_EXP_EP_DEM_ENR_INCL		A
					ON 	G.QUERY_GRP=A.QUERY_GRP
		 LEFT JOIN		&tmp.._PAT_EVENT_EPISODES				B
					ON 	A.PatID=B.PatID
				  AND	G.EVENT_GRP=B.EVENT_GRP
			ORDER BY G.GROUP
						, A.EXPOS_PERIOD_STRT_DT		
						, A.EXPOS_PERIOD_END_DT
						, A.PatID
						, A.EXPOSURE_DT
						, B.EVENT_DT
	
				;
		QUIT;

* Create flags for events in comparison and risk windows;
* Create variable for number days enrolled after exposure. ;
* Set CENSORED_FLAG=1 when enrollment ends within a comparison or risk window;
* The variable can be used to censor events;


DATA	&tmp.._PAT_EXPOS_OUTCOMES_FLAGS;
	 SET &tmp.._PAT_EXPOS_OUTCOMES;
	   BY GROUP EXPOS_PERIOD_STRT_DT EXPOS_PERIOD_END_DT  PatID EXPOSURE_DT EVENT_DT;
	     RETAIN CNT_IN_COMP_OR_RISK_WND;

	    IF FIRST.EXPOSURE_DT THEN
						DO;
			CNT_IN_COMP_OR_RISK_WND=0;
						END;

IN_COMP_WND=(MIN_DAYS_FR_INDEX_FOR_COMP_WND <=DAYS_FROM_EXPOS_TO_EVENT <=MAX_DAYS_FR_INDEX_FOR_COMP_WND
						AND DAYS_FROM_EXPOS_TO_EVENT NE .
						);
IN_RISK_WND=(MIN_DAYS_FR_INDEX_FOR_RISK_WND <=DAYS_FROM_EXPOS_TO_EVENT <=MAX_DAYS_FR_INDEX_FOR_RISK_WND
						AND DAYS_FROM_EXPOS_TO_EVENT NE .
						);

	CNT_IN_COMP_OR_RISK_WND=CNT_IN_COMP_OR_RISK_WND+MAX(IN_COMP_WND,IN_RISK_WND);

					CENSORED_FLAG=1;  

IF DAYS_ENR_AFTER_EXPOS > = MAX_DAYS_EVENT_WIND
THEN DO; CENSORED_FLAG=0; DAYS_ENR_AFTER_EXPOS=MAX_DAYS_EVENT_WIND; END;

* For event in comparison or risk window output only the very first encounter;
* For exposure episode with no event in in comparison or risk window output a single (last) record;
* The later will be used later to calculate number of exposed patients in exposure period;

IF   		CNT_IN_COMP_OR_RISK_WND=1
	OR	(CNT_IN_COMP_OR_RISK_WND=0 AND LAST.EXPOSURE_DT) 
THEN OUTPUT;

	FORMAT EXPOS_PERIOD_STRT_DT EXPOSURE_DT EVENT_DT DATE9.;
RUN;

	TITLE1 "Distribution of DAYS_FROM_EXPOS_TO_EVENT";
	TITLE2  "Stratified by GROUP EXPOS_PERIOD_STRT_DT IN_COMP_WND IN_RISK_WND";

	PROC MEANS DATA=&tmp.._PAT_EXPOS_OUTCOMES_FLAGS MIN  MAX ;
	  CLASS GROUP EXPOS_PERIOD_STRT_DT IN_COMP_WND IN_RISK_WND;
	  VAR DAYS_FROM_EXPOS_TO_EVENT;
	 RUN;

	TITLE1 "Distribution of DAYS_ENR_AFTER_EXPOS.";
	TITLE2  "Stratified by GROUP EXPOS_PERIOD_STRT_DT CENSORED_FLAG.";

	PROC MEANS DATA=&tmp.._PAT_EXPOS_OUTCOMES_FLAGS MIN  MAX ;
	  CLASS GROUP EXPOS_PERIOD_STRT_DT CENSORED_FLAG;
	  VAR DAYS_ENR_AFTER_EXPOS;
	 RUN;

	  * Create Final datasets;

      * Create patient exposure episode level dataset in DPLocal library ;
	  * Keep at a data partner site;

	 PROC SQL;
 		CREATE TABLE DPLocal.&out_tbl_prefix._PAT_EVNT AS
		 	SELECT  GROUP
						,B.CODE_DESC					AS STUDY_GROUP_DESC
						,EXPOSURE_PERIOD_CD
						,EXPOS_PERIOD_STRT_DT 
						,EXPOS_PERIOD_END_DT  
						,PatID 
 			   			,DAYS_FROM_EXPOS_TO_EVENT 
						,AGE_GROUP  
					    ,AGE_GRP_DSC 
						,SEX 
						,IN_COMP_WND
						,IN_RISK_WND
						,CENSORED_FLAG 
				  		,DAYS_ENR_AFTER_EXPOS
						,INPUT(COMPRESS("&DateStamp."),DATE9.) 	FORMAT=DATE9. AS  CREATE_TBL_DT 
       			FROM 			&tmp.._PAT_EXPOS_OUTCOMES_FLAGS 		A
				INNER JOIN 	&LKP_FOR_ALL_CODES.								B
							ON 	A.GROUP=B.CODE_VALUE
						  AND 	B.COLUMN_NAME="GROUP"
					ORDER BY  GROUP
									,EXPOS_PERIOD_STRT_DT 
									,EXPOS_PERIOD_END_DT  
									,PatID 
			 			   			,DAYS_FROM_EXPOS_TO_EVENT
				;
	   QUIT;
 
	 
      * Create summary exposure-event level dataset in DPLocal library ;
	  * Keep at data partner site;

	 PROC SQL;
 		CREATE TABLE DPLocal.&out_tbl_prefix._SUM_EXPS_EVNT AS
		 	SELECT  GROUP
						,STUDY_GROUP_DESC
						,EXPOSURE_PERIOD_CD
						,EXPOS_PERIOD_STRT_DT 
						,EXPOS_PERIOD_END_DT
						,AGE_GROUP  
					    ,AGE_GRP_DSC 
						,SEX
 			   			,DAYS_FROM_EXPOS_TO_EVENT
				  		,DAYS_ENR_AFTER_EXPOS 
						,COUNT(PatID)											AS CNT_EXP_EPISODES 
						,SUM(IN_COMP_WND) 								AS CNT_IN_COMP_WND
						,SUM(IN_RISK_WND)	 								AS CNT_IN_RISK_WND
						,SUM(CENSORED_FLAG)								AS CNT_CENSORED_EVENTS 
						,MAX(CREATE_TBL_DT)	FORMAT=DATE9.	AS CREATE_TBL_DT

       			FROM 			DPLocal.&out_tbl_prefix._PAT_EVNT 		
					GROUP BY  GROUP
									,STUDY_GROUP_DESC
									,EXPOSURE_PERIOD_CD
									,EXPOS_PERIOD_STRT_DT 
									,EXPOS_PERIOD_END_DT
									,AGE_GROUP  
								    ,AGE_GRP_DSC 
									,SEX
			 			   			,DAYS_FROM_EXPOS_TO_EVENT
							  		,DAYS_ENR_AFTER_EXPOS  
				;
	   QUIT;

	   * Save exposure episodes that were excluded by various reasons;

	DATA DPLocal.&out_tbl_prefix._PAT_EXPS_EXCL;
	  SET &tmp.._PAT_EXPOSURE_EPISODES_EXCL
 			&tmp.._PAT_EXP_EP_DEM_ENR_EXCL
			;
   RUN;

   * Save adverse events that were excluded by various reasons;

  DATA DPLocal.&out_tbl_prefix._PAT_EVNT_EXCL;
	  SET &tmp.._PAT_EVENT_EPISODES_EXCL;
   RUN;
      
  * Create summary dataset on event level in MSOC library;
  * To send to MSOC;

	   DATA MSOC.&out_tbl_prefix._SUM_EVNT;
	     SET DPLocal.&out_tbl_prefix._SUM_EXPS_EVNT;

		 * 20130530. Changed to keep only events that are either in comparison or risk windows;
		*  IF DAYS_FROM_EXPOS_TO_EVENT NE .;
		 IF CNT_IN_COMP_WND >0 OR CNT_IN_RISK_WND >0;
		RUN;
	
	* Create summary dataset on exposure period level in MSOC library;
    * To send to MSOC ;

	 PROC SQL;
 		CREATE TABLE MSOC.&out_tbl_prefix._SUM_EXPS_PERIOD AS
		 	SELECT  GROUP
						,STUDY_GROUP_DESC
						,EXPOSURE_PERIOD_CD
						,EXPOS_PERIOD_STRT_DT 
						,EXPOS_PERIOD_END_DT
						,AGE_GROUP  
					    ,AGE_GRP_DSC 
						,SEX
						,SUM(CNT_EXP_EPISODES)							AS CNT_EXP_EPISODES 
						,SUM(CNT_IN_COMP_WND) 						AS CNT_IN_COMP_WND
						,SUM(CNT_IN_RISK_WND)	 						AS CNT_IN_RISK_WND
						,SUM(CNT_CENSORED_EVENTS)					AS CNT_CENSORED_EVENTS 
						,MIN(DAYS_FROM_EXPOS_TO_EVENT) 		AS MIN_DAYS_FROM_EXPOS_TO_EVENT
 			   			,MAX(DAYS_FROM_EXPOS_TO_EVENT) 		AS MAX_DAYS_FROM_EXPOS_TO_EVENT
				  		,MIN(DAYS_ENR_AFTER_EXPOS) 				AS MIN_DAYS_ENR_AFTER_EXPOS
						,MAX(DAYS_ENR_AFTER_EXPOS) 				AS MAX_DAYS_ENR_AFTER_EXPOS
						,MAX(CREATE_TBL_DT)	FORMAT=DATE9.	AS CREATE_TBL_DT

       			FROM 			DPLocal.&out_tbl_prefix._SUM_EXPS_EVNT	
					GROUP BY  GROUP
									,STUDY_GROUP_DESC
									,EXPOSURE_PERIOD_CD
									,EXPOS_PERIOD_STRT_DT 
									,EXPOS_PERIOD_END_DT
									,AGE_GROUP  
								    ,AGE_GRP_DSC 
									,SEX
				;
	   QUIT;

	   
	 PROC SQL;
 		CREATE TABLE MSOC.&out_tbl_prefix._SUM_EXPS_PERIOD_FIN AS
		 	SELECT  MAX("&DPID.&SITEID.")							AS DPID_SITEID				
						,GROUP
						,STUDY_GROUP_DESC
						,EXPOSURE_PERIOD_CD
						,EXPOS_PERIOD_STRT_DT 
						,EXPOS_PERIOD_END_DT
						,SUM(CNT_EXP_EPISODES)							AS CNT_EXP_EPISODES 
						,SUM(CNT_IN_COMP_WND) 						AS CNT_IN_COMP_WND
						,SUM(CNT_IN_RISK_WND)	 						AS CNT_IN_RISK_WND
						
						,MAX(CREATE_TBL_DT)	FORMAT=DATE9.	AS CREATE_TBL_DT

       			FROM 			DPLocal.&out_tbl_prefix._SUM_EXPS_EVNT		
					GROUP BY  GROUP
									,STUDY_GROUP_DESC
									,EXPOSURE_PERIOD_CD
									,EXPOS_PERIOD_STRT_DT 
									,EXPOS_PERIOD_END_DT
				;
	   QUIT;

	   
data _NULL_;
   temp=DATETIME();
   seconds=temp-&start.;
   hours=int(seconds/3600);
   minutes=int((seconds-hours*3600)/60);
   seconds2=int((seconds-hours*3600-minutes*60));
   call symput("STOP",temp);
   call symput("hours",put(hours,4.0));
   call symput("minutes",put(minutes,2.0));
   call symput("seconds",put(seconds2,2.0));
   PUT _ALL_;
run;

%PUT TOTAL RUN TIME was &hours. h &minutes. m &seconds. s; 
   
data signature;
   DPID="&DPID.";                                    
   SITEID="&SITEID.";                                
   REQUESTID="&REQUESTID.";                            
   format Start_Time Stop_Time datetime21.2; 
   Start_Time=trim(left(&START.));                                
   Stop_Time=trim(left(&STOP.));                                  
   format Execution_time $20.; 
   Execution_time="&hours. h &minutes. m &seconds. s"; 
   ENROLGAP="&ENROLGAP.";
   TMP="&TMP.";
   QUERYFROM="&QUERYFROMc.";                            
   QUERYTO="&QUERYTOc.";
   QUERYFILE="&QUERYFILE.";
   INCQUERYFILE="&INCQUERYFILE.";
   EVENTFILE="&EVENTFILE.";
   INCEVENTFILE="&INCEVENTFILE.";
   LKP_FOR_ALL_CODES_FILE="&LKP_FOR_ALL_CODES_FILE.";
   EXPOSURE_PERIODS_FILE="&EXPOSURE_PERIODS_FILE.";
   GROUPS_MAPPING_FILE="&GROUPS_MAPPING_FILE.";
   QUERY_CODES_FILE="&QUERY_CODES_FILE.";
   EVENT_CODES_FILE="&EVENT_CODES_FILE."; 
   AGE_GROUPS_FILE="&AGE_GROUPS_FILE.";
	;
   output;
run;

proc transpose data=signature out=_signature(rename=_NAME_=Var rename=COL1=VALUE);
   var _ALL_;
run;

DATA MSOC.&out_tbl_prefix._SIGNATURE;
 SET _signature;
 RUN;

  PROC PRINTTO LOG=LOG PRINT=PRINT;
 RUN;

 %MEND;


